stm8/    ; кодировка файла должна быть ASCII или UTF-8 без BOM

; Плата STM8S103F3P6 -  Аппаратный звуковой сигнал на ножку PD4 (BEEP)
;                                         _ _ _ _ _ _ _ _ _ _ _ _  _ _ _ _ _ _ _ _ _ _ _ _
;                                      __|                       \/                       |__ 
;    |\   ,---------------------------|__| PD4 (HS) TIM2_CH1            TIM2_CH2 (HS) PD3 |__|
;    | \  |                            1 |    UART1_CK  BEEP            AIN4   ADC_ETR    |20 
;    |  \---,                          __|                                                |__ 
;    |  |   |     USB-TTL_UART RX <-- |__| PD5 (HS) AIN5                AIN3     (HS) PD2 |__|
;    HCM1212A                          2 |    UART1_TX                  [TIM2_CH3]        |19 
;    |  |   |                          __|                                                |__
;    |  |   |     USB-TTL_UART TX --> |__| PD6 (HS) AIN6                SWIM     (HS) PD1 |__|-SWIM
;    |  /---'                          3 |    UART1_RX                                    |18 
;    | /  |                            __|                                                |__
;    |/  GND                   NRST---|__| NRST                         SPI_MISO (HS) PC7 |__|
;                                      4 |                              [TIM1_CH2]        |17
    MOTOROLA
    .NOLIST
    #include "STM8S103F3P.inc"
    .LIST
    
    #define F_OSC 8000000   ; RC osc. 16 MHz / 2 (default)
    
    BYTES            ; далее следуют 8-разрядные метки и адреса
    segment byte at 0000 'ram_var' ; переменные
morse_counter   DS.B 1
morse_sequence  DS.B 1
morse_value     DS.B 1
dot_lenght      DS.B 2
delay_counter   DS.B 1

    WORDS            ; далее следуют 16-разрядные метки и адреса
    segment byte at 8000 'main_flash' ; сегмент с кодом расположен по адресу $8000 (начало памяти FLASH)
reset:    
; настраиваем аппаратный BEEP
    ld        A, #%00000010  ; 128000 / 8 / 16 = 1000 Hz
    ld        BEEP_CSR, A
; настроим длительность точки
    ldw    X, #{F_OSC/160}   ; 8000000 / 160 = 50000
    ldw    dot_lenght, X
; настраиваем UART 9600 1N8
    mov    UART1_BRR1, #13
    mov    UART1_CR2, #%00001100 

main_cycle:



UART_rx_wait:
; ждем байт
    btjf  UART1_SR, #5, *  
    ld    A, UART1_DR
    
    cp    A, #{' '} ; если код символа ASCII меньше $20
;    cp    A, #$20 ;
    jrult  UART_tx_ERR ; ошибка
    cp    A, #{'_'} ; если код символа ASCII больше $5F
;    cp    A, #$5F ;
    jrugt  UART_tx_ERR ; ошибка
; если код символа ASCII в диапазоне $20...$5F
    sub   A, #{' '} ; выравниваем принятый символ по началу таблицы
;    sub   A, #$20 ;
    ldw   Y, #0
    ld    YL, A
    ld    A, (ascii_morse_table,Y)
	ld    morse_value, A
    call   morse
	
UART_tx_OK:
; отправляем OK
    mov    UART1_DR, #{'O'} ; 
    btjf  UART1_SR, #7, * 
    mov    UART1_DR, #{'K'} ; 
    btjf  UART1_SR, #7, * 
    jra    UART_rx_wait
    
UART_tx_ERR:
; отправляем ERR
    mov    UART1_DR, #{'E'} ; 
    btjf  UART1_SR, #7, * 
    mov    UART1_DR, #{'R'} ; 
    btjf  UART1_SR, #7, * 
    mov    UART1_DR, #{'R'} ; 
    btjf  UART1_SR, #7, * 
    jra    UART_rx_wait

    jp    main_cycle


; последовательность знаков в morse_value
morse: 
; минимальный временной интервал - длительность точки (dot_delay или интервал)
; знаки:
;   точка (dot) - состоит из эвукового сигнала длительностью интервал и паузы длительностью интервал
;   тире (dash) - состоит из эвукового сигнала длительностью три интервала и паузы длительностью интервал
; символы:
;   состоят из знаков, пауза после последнего знака удлиняется до трех (1+2) интервалов
; слова:
;   состоят из символов, пауза после последнего символа удлиняется до семи (1+2+4) интервалов
; кодировка:
;    все разряды равны 0 ([7:0]=00000000), точки/тире не обрабатываютя, формируется пауза длительностью четыре интервала (разделитель слов)
;    для символов старшие три разряда [7:5]=n кодируют количество знаков в символе (от 1 до 6 знаков) 
;                        добавляется разделитель символов (пауза длительностью два интервала)
;    для знаков старшие три разряда [7:5]=000, разряды [4:3]=n кодируют количество знаков (от 1 до 3 знака)
;                        пауза-разделитель символов не добавляется
;    младшие разряды (от 1 до 6, выровненные по 0-му разряду) кодируют знаки (0-точка, 1-тире) в обратном порядке 
;        извлечение знаков через пощаговый сдвиг вправо через Carry флаг
             ; [0, 0,  0,   0,   0,   0,   0,   0 ] - пауза длительностью четыре интервала (разделитель слов)
             ; [0, 0,  0,   0,   0,   i,   i,   i ] - разряды iii игнорируются,  пауза-разделитель символов не добавляется
             ; [0, 0,  0,   0,   1,   i,   i,  1st] - один знак, разряды ii игнорируются,  пауза-разделитель символов не добавляется
             ; [0, 0,  0,   1,   0,   i,  2nd, 1st] - два знака, разряд i игнорируется, пауза-разделитель символов не добавляется 
             ; [0, 0,  0,   1,   1,  3th, 2nd, 1st] - три знака, пауза-разделитель символов не добавляется 
             ; [0, 0,  1,   i,   i,   i,   i,  1st] - один знак, разряды iiii игнорируются, с добавлением паузы длительностью два интервала (разделитель символов)
             ; [0, 1,  0,   i,   i,   i,  2nd, 1st] - два знака, разряды iii игнорируются, с добавлением паузы длительностью два интервала (разделитель символов)
             ; [0, 1,  1,   i,   i,  3th, 2nd, 1st] - три знака, разряды ii игнорируются, с добавлением паузы длительностью два интервала (разделитель символов)
             ; [1, 0,  0,   i,  4th, 3th, 2nd, 1st] - четыре знака, разряд i игнорируется, с добавлением паузы длительностью два интервала (разделитель символов)
             ; [1, 0,  1,  5th, 4th, 3th, 2nd, 1st] - пять знаков, с добавлением паузы длительностью два интервала (разделитель символов)
             ; [1, 1, 6th, 5th, 4th, 3th, 2nd, 1st] - шесть знаков, с добавлением паузы длительностью два интервала (разделитель символов)
; одним байтом обрабатывается симол из 1-6 знаков
; двумя байтами обрабатывается диграф или триграф из 7-9 знаков 
    ld   A, morse_value
    cp    A, #%00000000
    jreq  four_interval  ; если morse_value = 0 отрабатываем только паузу длительностью 4 интервала
    cp    A, #%00001000
    jrult  four_interval_exit ; если 0 < morse_value < 16 выходим
    cp    A, #%00100000
    jrult  morse_without_postpause ;
morse_sequence_with_pause:
; если morse_value >= 32 отрабатываем morse с постпаузой
    ld    morse_sequence, A
	swap  A
	srl   A
    and   A, #%00000111
    ld    morse_counter, A
    cp    A, #%00000111
    jreq  morse_with_pause_is_7
    jra    morse_with_pause_go
morse_with_pause_is_7:
	mov   morse_counter, #%00000110 
morse_with_pause_go:
    callr    morse_cycle
    mov     delay_counter, #2
    callr   dot_delay
    ret ; from morse
	
; если morse_value < 32 отрабатываем morse с постпаузой
morse_without_postpause:
    mov   morse_sequence, morse_value
    ld    A, morse_value
	sll   A
	swap  A
    and   A, #%00000011
	ld    morse_counter, A
    callr    morse_cycle
    ret ; from morse
	
four_interval:   ; 4 паузы
    mov     delay_counter, #4
    callr   dot_delay
four_interval_exit:
    ret ; from morse
    
    
    
morse_cycle:
    srl    morse_sequence   
    jrc    dash
    jra    dot
morse_cycle_decrement:
    dec     morse_counter
    jrne    morse_cycle
    mov     delay_counter, #3
    callr   dot_delay
    ret ; morse

    
dash: ; 3 dot_delay BEEP and 1 dot_delay pause
; включаем светодиод
    bset PB_DDR,#5 
;    bset PB_CR1,#5 
; включаем звуковой сигнал
    bset    BEEP_CSR, #5    
    mov     delay_counter, #3
    callr   dot_delay
; выключаем светодиод
    bres PB_DDR,#5 
;    bres PB_CR1,#5 
; выключаем звуковой сигнал
    bres    BEEP_CSR, #5
    mov     delay_counter, #1
    callr   dot_delay
    jra     morse_cycle_decrement
    
dot: ; 1 dot_delay BEEP and 1 dot_delay pause
; включаем светодиод
    bset PB_DDR,#5 
;    bset PB_CR1,#5 
; включаем звуковой сигнал
    bset    BEEP_CSR, #5    
    mov     delay_counter, #1
    callr   dot_delay
; выключаем светодиод
    bres PB_DDR,#5 
;    bres PB_CR1,#5 
; выключаем звуковой сигнал
    bres    BEEP_CSR, #5
    mov     delay_counter, #1
    callr   dot_delay
    jra     morse_cycle_decrement
    
dot_delay:    
    ldw     X, dot_lenght
unit_delay:
    decw    X
    jrne    unit_delay
    dec     delay_counter
    jrne    dot_delay
    ret
    
    
    
ascii_morse_table:
; ASCII         $20        $21        $22        $23
; ASCII       пробел        !          "          #
    dc.b    %00000000, %11110011, %11010010, %00000000 ;
; Морзе                  --..--     .-..-.        
; Морзе                 восклиц.   кавычки        

; ASCII         $24        $25        $26        $27
; ASCII          $          %          &          '
    dc.b    %00000000, %00000000, %00000000, %11011110
; Морзе                                        .----.
; Морзе                                       апостроф

; ASCII         $28        $29        $2A        $2B
; ASCII          (          )          *          +
    dc.b    %11101101, %11101101, %00000000, %00000000
; Морзе       -.--.-     -.--.- 
; Морзе       скобка     скобка

; ASCII         $2C        $2D        $2E        $2F
; ASCII          ,          -          .          /
    dc.b    %11101010, %11100001, %11000000, %10101001
; Морзе       .-.-.-     -....-     ......      -..-.
; Морзе       запятая     тире       точка   косая черта

; ASCII         $30        $31        $32        $33
; ASCII          0          1          2          3
    dc.b    %10111111, %10111110, %10111100, %10111000 
; Морзе        -----      .----      ..---      ...--
; Морзе          0          1          2          3

; ASCII         $34        $35        $36        $37
; ASCII          4          5          6          7
    dc.b    %10110000, %10100000, %10100001, %10100011 
; Морзе        ....-      .....      -....      --...
; Морзе          4          5          6          7
    
; ASCII         $38        $39        $3A        $3B
; ASCII          8          9          :          ;
    dc.b    %11100111, %10101111, %11000111, %11010101
; Морзе        ---..      ----.     ---...     -.-.-.
; Морзе          8          9          :          ;
        
; ASCII         $3C        $3D        $3E        $3F   
; ASCII          <          =          >          ?    
    dc.b    %00000000, %10110001, %00000000, %11001100 
; Морзе                  -...-                ..--..
; Морзе                  раздел.               вопрос.

 
; ASCII         $40        $41        $42        $43
; ASCII          @          A          B          C                                               
    dc.b    %11010110, %01000010, %10000001, %10000101
; Морзе       .--.-.       .-        -...       -.-.
; Морзе          @          A          B          C
; Морзе                     А          Б          Ц

; ASCII         $44        $45        $46        $47
; ASCII          D          E          F          G     
    dc.b    %01100001, %00100000, %10000100, %01100011 
; Морзе         -..         .        ..-.        --.
; Морзе          D          E          F          G  
; Морзе          Д          Е(Ё)       Ф          Г

; ASCII         $48        $49        $4A        $4B
; ASCII          H          I          J          K    
    dc.b    %10000000, %01000000, %10001110, %01100101
; Морзе         ....        ..       .---        -.-
; Морзе          H          I          J          K
; Морзе          Х          И          Й          К
        
; ASCII         $4C        $4D        $4E        $4F
; ASCII          L          M          N          O        
    dc.b    %10000010, %01000011, %01000001, %01100111
; Морзе         .-..        --         -.        ---
; Морзе          L          M          N          O
; Морзе          Л          М          Н          О

; ASCII         $50        $51        $52        $53
; ASCII          P          Q          R          S
    dc.b    %10000110, %10011011, %01100010, %01100000 
; Морзе         .--.       --.-       .-.        ...
; Морзе          P          Q          R          S
; Морзе          П          Щ          Р          С

; ASCII         $54        $55        $56        $57
; ASCII          T          U          V          W
    dc.b    %00100001, %01100100, %10001000, %01100110 
; Морзе          -         ..-        ...-       .--     
; Морзе          T          U          V          W
; Морзе          Т          У          Ж          В
        
; ASCII         $58        $59        $5A        $5B
; ASCII          X          Y          Z          [
    dc.b    %10001001, %10001101, %10000011, %11101101 
; Морзе         -..-      -.--        --..     -.--.- 
; Морзе          X          Y          Z       скобка 
; Морзе          Ь(Ъ)       Ы          З
        
; ASCII         $5C        $5D        $5E        $5F
; ASCII          \          ]          ^          _
    dc.b    %00000000, %11101101, %00000000, %10110001
; Морзе                  -.--.-                -...- 
; Морзе                  скобка                раздел.
;
; не закодированы:
; Ч(---.), Ш(----), Ъ(--.--), Э(..-..), Ю(..--), Я(.-.-),
; конец связи (..-..-), знак окончания передачи (.-.-.)
; ждать (.-...), понял (...-.), начало действия (.-.-.-.-.-.-)
;


    

    end        ; перед end должен быть хотя бы один пробел или табуляция, после end хотябы одна строка
;   .        Ee     Ее Ёё
;   -        Tt     Тт
;
;   ..       Ii     Ии
;   .-       Aa     Аа
;   -.       Nn     Нн
;   --       Mm     Мм
;
;   ...      Ss     Сс
;   ..-      Uu     Уу
;   .-.      Rr     Рр
;   .--      Ww     Вв
;   -..      Dd     Дд
;   -.-      Kk     Кк     also used for [Invitation to Transmit]
;   --.      Gg     Гг
;   ---      Oo     Оо
;
;   ....     Hh     Хх
;   ...-     Vv     Жж
;   ..-.     Ff     Фф
;   ..--            Юю
;   .-..     Ll     Лл
;   .-.-            Яя    [AA] digraph     UNKNOWN STATION
;   .--.     Pp     Пп
;   .---     Jj     Йй
;   -...     Bb     Бб
;   -..-     Xx     Ьь Ъъ
;   -.-.     Cc     Цц
;   -.--     Yy     Ыы
;   --..     Zz     Зз
;   --.-     Qq     Щщ
;   ---.            Чч
;   ----            Шш
; 
;   .----    1
;   ..---    2
;   ...--    3
;   ....-    4
;   .....    5
;   -....    6
;   --...    7
;   ---..    8
;   ----.    9
;   -----    0
;   ..-..           Ээ
;
;   ..-.-    [INT] trigraph INTERROGATIVE ACP124 ACP131 - military network question marker


;   -..-.    Slash/Fraction Bar [/]
;   -.--.    Parenthesis (Open)
;   .-...    Ampersand (or "Wait") [&]      [AS] digraph
;   -...-    [BT] digraph BREAK ITU-R M.1172 ITU-R M.1677-1         Same as character Double Dash = or --
;   .-.-.    Plus sign [+] 
;   .-.-.    New Page Signal  [AR] digraph   Message separator   Single-line display may use printed "+"
;   -.-.-    Starting Signal
;   ...-.    Understood
;   .--.-    À, à   Shared by À, Å
;
;   .. .. .. точка
;   .-.-.-   запятая
;
;   .--.-.          Ъъ

;   .-.-.-   Period [.]
;   --..--   Comma [,]
;   ..--..   Question Mark [?]    [UD] digraph
;   .----.   Apostrophe [']
;   -.-.--   Exclamation Point [!]   [KW] digraph
;   -.--.-   Parenthesis (Close)
;   ---...   Colon [:] 
;   -.-.-.   Semicolon [;] 
;   -....-   Hyphen, Minus Sign [-] 
;   ..--.-   Underscore [_] 
;   .-..-.   Quotation mark ["]  
;   .--.-.   At Sign [@]               [AC] digraph
;   ...-.-   End of work
;

;   ...-..-  Dollar sign [$]           [SX] digraph

;   ........ Error/correction     [HH] digraph or [EEEEEEEE]

;   ...---...   [SOS] trigraph в отличие от SOS отправляется без символьных интервалов
